# -*- coding: utf-8 -*-
import reusables
import re

from box import Box

from fastflix.plugins.common.helpers import generate_filters, Command
from fastflix.plugins.common.audio import build_audio
from fastflix.plugins.common.subtitles import build_subtitle


def build(
    source,
    video_track,
    bitrate=None,
    crf=None,
    start_time=0,
    duration=None,
    preset="fast",
    audio_tracks=(),
    subtitle_tracks=(),
    disable_hdr=False,
    side_data=None,
    x265_params=None,
    intra_encoding=False,
    max_mux="default",
    extra="",
    **kwargs,
):
    filters = generate_filters(disable_hdr=disable_hdr, **kwargs)
    audio = build_audio(audio_tracks)
    subtitles = build_subtitle(subtitle_tracks)

    ending = "dev/null && \\"
    if reusables.win_based:
        ending = "NUL"

    if not side_data:
        side_data = Box(default_box=True)

    beginning = (
        f'"{{ffmpeg}}" -y '
        f'-i "{source}" '
        f' {f"-ss {start_time}" if start_time else ""}  '
        f'{f"-to {duration}" if duration else ""} '
        f"{extra} "
        f"-map 0:{video_track} "
        # "-pix_fmt yuv420p10le "
        f"-c:v libx265 "
        f'{f"-vf {filters}" if filters else ""} '
        # f'{"-pix_fmt yuv420p" if force420 else ""} '
    )

    beginning = re.sub("[ ]+", " ", beginning)

    if max_mux and max_mux != "default":
        beginning += f"-max_muxing_queue_size {max_mux} "

    if not x265_params:
        x265_params = []

    if not disable_hdr:
        if side_data.color_primaries == "bt2020":
            x265_params.extend(
                ["hdr-opt=1", "repeat-headers=1", "colorprim=bt2020", "transfer=smpte2084", "colormatrix=bt2020nc"]
            )

        if side_data.master_display:
            x265_params.append(
                "master-display="
                f"G{side_data.master_display.green}"
                f"B{side_data.master_display.blue}"
                f"R{side_data.master_display.red}"
                f"WP{side_data.master_display.white}"
                f"L{side_data.master_display.luminance}"
            )

        if side_data.cll:
            x265_params.append(f"max-cll={side_data.cll}")

    if intra_encoding:
        x265_params.append("keyint=1")

    if x265_params:
        beginning += '-x265-params "{}" '.format(":".join(x265_params))

    if side_data.cll:
        pass

    extra_data = "-map_chapters 0 "  # -map_metadata 0 # safe to do for rotation?

    if bitrate:
        command_1 = f'{beginning}:pass=1 -passlogfile "<tempfile.1.log>" -b:v {bitrate} -an -f mp4 {ending}'
        command_2 = (
            f'{beginning}:pass=2 -passlogfile "<tempfile.1.log>" '
            f'-b:v {bitrate} -preset {preset} {audio} {subtitles} {extra_data} "{{output}}"'
        )
        return [
            Command(command_1, ["ffmpeg", "output"], False, name="First pass bitrate", exe="ffmpeg"),
            Command(command_2, ["ffmpeg", "output"], False, name="Second pass bitrate", exe="ffmpeg"),
        ]

    elif crf:
        command = f'{beginning} -crf {crf} -preset {preset} {audio} {subtitles} {extra_data} "{{output}}"'
        return [Command(command, ["ffmpeg", "output"], False, name="Single pass CRF", exe="ffmpeg")]

    else:
        return []



# -*- coding: utf-8 -*-
import os
import sys
import importlib.machinery

try:
    # PyInstaller creates a temp folder and stores path in _MEIPASS
    # noinspection PyUnresolvedReferences
    base_path = sys._MEIPASS
    pyinstaller = True
except AttributeError:
    base_path = os.path.abspath(".")
    pyinstaller = False

# This is required to keep LGPL libraries truly dynamically linked when built into a binary
if os.getenv("SHIBOKEN2"):
    importlib.machinery.SourceFileLoader("shiboken2", os.getenv("SHIBOKEN2")).load_module()
if os.getenv("PYSIDE2"):
    PySide2 = importlib.machinery.SourceFileLoader("PySide2", os.getenv("PYSIDE2")).load_module()

from PySide2 import QtWidgets, QtCore, QtGui
from PySide2.QtCore import Qt
from PySide2 import __version__ as pyside_version

QtWidgets.QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling, True)

main_width = 800


def message(msg, parent=None):
    sm = QtWidgets.QMessageBox(parent=parent)
    sm.setText(msg)
    sm.setStandardButtons(QtWidgets.QMessageBox.Ok)
    sm.exec_()


def error_message(msg, details=None, traceback=False, parent=None):
    em = QtWidgets.QMessageBox(parent=parent)
    em.setText(msg)
    if details:
        em.setDetailedText(details)
    elif traceback:
        import traceback

        em.setDetailedText(traceback.format_exc())
    em.setStandardButtons(QtWidgets.QMessageBox.Ok)
    em.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from subprocess import Popen, PIPE, run, STDOUT

import reusables

from fastflix.shared import QtCore

logger = logging.getLogger("fastflix")

__all__ = ["Worker"]


class Worker(QtCore.QThread):
    def __init__(self, app, command="", target=None, params=None, cmd_type="convert"):
        super(Worker, self).__init__(app)
        self.app = app
        self.command = command
        self.target = target
        self.params = params if params else {}
        self.cmd_type = cmd_type
        self.process = None
        self.killed = False

    def run(self):
        if self.command:
            logger.info(f"Running command: {self.command}")
            self.process = self.start_exec()
            while True:
                next_line = self.process.stdout.readline().decode("utf-8")
                if not next_line:
                    if self.process.poll() is not None:
                        break
                    else:
                        continue
                logger.debug(f"ffmpeg - {next_line}")
            return_code = self.process.poll()
            if self.killed:
                return self.app.cancelled.emit()
        else:
            try:
                return_code = self.target(**self.params)
            except Exception as err:
                logger.error(f"Could not run target {self.target}: {err}")
                return self.app.completed.emit(1)

        if self.cmd_type == "convert":
            self.app.completed.emit(return_code)
        elif self.cmd_type == "thumb":
            self.app.thumbnail_complete.emit()

    def start_exec(self):
        return Popen(self.command, stdin=PIPE, stdout=PIPE, stderr=STDOUT, shell=True)

    def is_alive(self):
        if not self.process:
            return False
        return True if self.process.poll() is None else False

    def kill(self):
        if self.process and self.is_alive():
            self.killed = True
            if reusables.win_based:
                run(f"TASKKILL /F /PID {self.process.pid} /T", stdin=PIPE, stdout=PIPE, stderr=PIPE)
            else:
                run(f"kill -9 {self.process.pid}", stdin=PIPE, stdout=PIPE, stderr=PIPE)
            return self.process.terminate()

    def __del__(self):
        self.kill()



# -*- coding: utf-8 -*-
import reusables
import re

from fastflix.plugins.common import helpers
from fastflix.plugins.common.audio import build_audio


def build(source, video_track, bitrate=None, crf=None, start_time=0, duration=None, audio_tracks=(), **kwargs):
    filters = helpers.generate_filters(**kwargs)
    audio = build_audio(audio_tracks)

    ending = "dev/null && \\"
    if reusables.win_based:
        ending = "NUL"

    beginning = (
        f'"{{ffmpeg}}" -y '
        f' {f"-ss {start_time}" if start_time else ""}  '
        f'{f"-t {duration}" if duration else ""} '
        f'-i "{source}" '
        f"-map 0:{video_track} "
        f"-c:v libaom-av1 -strict experimental "
        f'{f"-vf {filters}" if filters else ""} '
    )

    beginning = re.sub("[ ]+", " ", beginning)

    if bitrate:
        command_1 = f'{beginning} -passlogfile "<tempfile.1.log>" -b:v {bitrate} -pass 1 -an -f matroska {ending}'
        command_2 = f'{beginning} -passlogfile "<tempfile.1.log>" -b:v {bitrate} -pass 2 {audio} "{{output}}"'
        return [
            helpers.Command(command_1, ["ffmpeg", "output"], False, name="First Pass bitrate"),
            helpers.Command(command_2, ["ffmpeg", "output"], False, name="Second Pass bitrate"),
        ]
    elif crf:
        command_1 = f'{beginning} -b:v 0 -crf {crf} {audio} "{{output}}"'
        return [helpers.Command(command_1, ["ffmpeg", "output"], False, name="Single Pass CRF")]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pathlib import Path
import pkg_resources

from fastflix.shared import QtWidgets, QtGui
from fastflix.widgets.main import Main
from fastflix.widgets.about import About
from fastflix.widgets.logs import Logs


class Container(QtWidgets.QMainWindow):
    def __init__(self, data_path, work_path, **kwargs):
        super(Container, self).__init__()
        self.logs = Logs()
        self.about = None
        self.init_menu()
        main = Main(self, data_path, work_path, **kwargs)
        self.setCentralWidget(main)
        self.setMinimumSize(1200, 600)
        my_data = str(Path(pkg_resources.resource_filename(__name__, f"../data/icon.ico")).resolve())
        icon = QtGui.QIcon(my_data)
        self.setWindowIcon(icon)
        self.setWindowIcon(icon)

    def init_menu(self):
        exit_action = QtWidgets.QAction("&Exit", self)
        exit_action.setShortcut(QtGui.QKeySequence("Ctrl+Q"))
        exit_action.setStatusTip("Exit application")
        exit_action.triggered.connect(self.close)

        menubar = self.menuBar()
        file_menu = menubar.addMenu("&File")
        file_menu.addAction(exit_action)

        about_action = QtWidgets.QAction("&About", self)
        about_action.triggered.connect(self.show_about)

        log_action = QtWidgets.QAction("View &Logs", self)
        log_action.triggered.connect(self.show_logs)

        help_menu = menubar.addMenu("&Help")
        help_menu.addAction(log_action)
        help_menu.addAction(about_action)

    def show_about(self):
        self.about = About()
        self.about.show()

    def show_logs(self):
        self.logs.show()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fastflix',
 'fastflix.builders',
 'fastflix.plugins',
 'fastflix.plugins.av1',
 'fastflix.plugins.common',
 'fastflix.plugins.gif',
 'fastflix.plugins.hevc',
 'fastflix.plugins.svt_av1',
 'fastflix.plugins.vp9',
 'fastflix.widgets',
 'fastflix.widgets.panels']

package_data = \
{'': ['*'], 'fastflix': ['data/*', 'data/rotations/*']}

install_requires = \
['PySide2>=5.15.0,<6.0.0',
 'appdirs>=1.4.4,<2.0.0',
 'python-box[all]>=5.0.1,<6.0.0',
 'requests>=2.24.0,<3.0.0',
 'reusables>=0.9.5,<0.10.0',
 'ruamel.yaml>=0.16.10,<0.17.0',
 'shiboken2>=5.15.0,<6.0.0']

entry_points = \
{'console_scripts': ['fastflix = fastflix.__main__:main']}

setup_kwargs = {
    'name': 'fastflix',
    'version': '2.3.0',
    'description': 'Easy to use video encoder GUI wrapper',
    'long_description': None,
    'author': 'Chris Griffith',
    'author_email': 'chris@cdgriffith.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



# -*- coding: utf-8 -*-
import sys
import logging
from logging.handlers import SocketHandler
from pathlib import Path
from distutils.version import StrictVersion
from datetime import datetime
import os
import shutil
import traceback

try:
    import pkg_resources.py2_warn  # Needed for pyinstaller on 3.8
except ImportError:
    pass

try:
    from appdirs import user_data_dir
    from box import Box
    import reusables
    import requests

    from fastflix.version import __version__
    from fastflix.flix import ff_version, FlixError
    from fastflix.shared import QtWidgets, error_message, base_path, Qt
    from fastflix.widgets.container import Container
except ImportError as err:
    traceback.print_exc()
    print("Could not load FastFlix properly!", file=sys.stderr)
    input("Plese report this issue on https://github.com/cdgriffith/FastFlix/issues (press any key to exit)")
    sys.exit(1)

logger = logging.getLogger("fastflix")


def main():
    logging.basicConfig(level=logging.DEBUG, format="%(asctime)s - %(name)-6s  %(levelname)-8s %(message)s")
    socket_handler = SocketHandler("127.0.0.1", 19996)
    logger.addHandler(socket_handler)

    main_app = QtWidgets.QApplication(sys.argv)
    main_app.setStyle("fusion")
    main_app.setApplicationDisplayName("FastFlix")

    data_path = Path(user_data_dir("FastFlix", appauthor=False, roaming=True))
    ffmpeg_folder = Path(user_data_dir("FFmpeg", appauthor=False, roaming=True))
    data_path.mkdir(parents=True, exist_ok=True)
    log_dir = data_path / "logs"
    log_dir.mkdir(parents=True, exist_ok=True)

    ffmpeg = shutil.which("ffmpeg")
    if ffmpeg:
        ffmpeg = Path(ffmpeg)
    ffprobe = shutil.which("ffprobe")
    if ffprobe:
        ffprobe = Path(ffprobe)
    svt_av1 = shutil.which("SvtAv1EncApp")

    if ffmpeg_folder.exists():
        for file in ffmpeg_folder.iterdir():
            if file.is_file() and file.name.lower() in ("ffmpeg", "ffmpeg.exe"):
                ffmpeg = file
            if file.is_file() and file.name.lower() in ("ffprobe", "ffprobe.exe"):
                ffprobe = file
        if (not ffmpeg or not ffprobe) and (ffmpeg_folder / "bin").exists():
            for file in (ffmpeg_folder / "bin").iterdir():
                if file.is_file() and file.name.lower() in ("ffmpeg", "ffmpeg.exe"):
                    ffmpeg = file
                if file.is_file() and file.name.lower() in ("ffprobe", "ffprobe.exe"):
                    ffprobe = file

    logger.addHandler(logging.FileHandler(log_dir / f"flix_{datetime.now().isoformat().replace(':', '.')}"))

    config_file = Path(data_path, "fastflix.json")
    if not config_file.exists():
        config = Box({"version": __version__, "work_dir": str(data_path)})
        config.to_json(filename=config_file, indent=2)
    else:
        config = Box.from_json(filename=config_file)
        if StrictVersion(config.version) < StrictVersion(__version__):
            # do upgrade of config
            config.version = __version__
            config.to_json(filename=config_file, indent=2)
        if "ffmpeg" in config:
            ffmpeg = Path(config.ffmpeg)
        if "ffprobe" in config:
            ffprobe = Path(config.ffprobe)
        if "svt_av1" in config:
            svt_av1 = Path(config.svt_av1)
    work_dir = Path(config.get("work_dir", data_path))
    if not work_dir.exists():
        try:
            work_dir.mkdir(parents=True, exist_ok=True)
        except OSError as err:
            logger.error(
                f"Cannot use specified working directory {work_dir}"
                f" - Falling back to {data_path} due to error: {err}"
            )
            work_dir = data_path
            work_dir.mkdir(parents=True, exist_ok=True)

    if not ffmpeg or not ffprobe:
        qm = QtWidgets.QMessageBox
        if reusables.win_based:
            ret = qm.question(
                None,
                "FFmpeg not found!",
                f"<h2>FFmpeg not found!</h2>" f"<br> Automatically download FFmpeg?",
                qm.Yes | qm.No,
            )
            if ret == qm.Yes:
                try:
                    windows_download_ffmpeg(ffmpeg_folder)
                except Exception as err:
                    logger.exception("Could not download FFmpeg")
                    sys.exit(2)
                else:
                    ffmpeg = ffmpeg_folder / "bin" / "ffmpeg.exe"
                    ffprobe = ffmpeg_folder / "bin" / "ffprobe.exe"
            else:
                sys.exit(1)
        else:
            qm.question(
                None, "<h2>FFmpeg not found!</h2>", "Please download FFmpeg via your platform package manager", qm.Close
            )
            sys.exit(1)
    else:
        logger.info(f"Using ffmpeg {ffmpeg}")
        logger.info(f"Using ffprobe {ffprobe}")

    svt_av1_folder = Path(user_data_dir("SVT-AV1", appauthor=False, roaming=True))
    if not svt_av1 and svt_av1_folder.exists():
        svt_av1 = Path(svt_av1_folder, "SvtAv1EncApp.exe")

    if (not svt_av1 or not svt_av1.exists()) and reusables.win_based:
        qm = QtWidgets.QMessageBox
        ret = qm.question(
            None,
            "Download SVT-AV1",
            f"<h2>Would you like to download SVT-AV1?</h2>" f"<br> Will be placed in:" f"<br> {svt_av1_folder}",
            qm.Yes | qm.No,
        )
        if ret == qm.Yes:
            svt_av1_folder.mkdir(parents=True, exist_ok=True)
            try:
                download_svt_av1(svt_av1_folder)
            except Exception:
                logging.exception("Could not download newest SVT-AV1!")
                qm.question(None, "", f"Could not download SVT-AV1!", qm.Close)
                sys.exit(1)
            else:
                svt_av1 = Path(svt_av1_folder, "SvtAv1EncApp.exe")
    try:
        ffmpeg_version = ff_version(ffmpeg, throw=True)
        ffprobe_version = ff_version(ffprobe, throw=True)
    except FlixError:
        error_message("ffmpeg or ffmpeg could not be executed properly!")
        sys.exit(1)

    try:
        window = Container(
            ffmpeg=ffmpeg,
            ffprobe=ffprobe,
            ffmpeg_version=ffmpeg_version,
            ffprobe_version=ffprobe_version,
            svt_av1=svt_av1,
            source=sys.argv[1] if len(sys.argv) > 1 else "",
            data_path=data_path,
            work_path=work_dir,
        )
        window.show()
    except (Exception, BaseException, SystemError, SystemExit):
        logger.exception("HARD FAIL: Unexpected error")
        sys.exit(1)
    sys.exit(main_app.exec_())


def download_svt_av1(svt_av1_folder):
    logger.info(f"Downloading SVT-AV1 to {svt_av1_folder}")

    if reusables.win_based:
        svt_av1_releases = requests.get(f"https://api.github.com/repos/OpenVisualCloud/SVT-AV1/releases").json()
        svt_av1_assets = requests.get(svt_av1_releases[0]["assets_url"]).json()
        for asset in svt_av1_assets:
            reusables.download(asset["browser_download_url"], save_dir=svt_av1_folder)


def windows_download_ffmpeg(ffmpeg_folder):
    ffmpeg_folder.mkdir(exist_ok=True)
    url = "https://ffmpeg.zeranoe.com/builds/win64/static/ffmpeg-latest-win64-static.zip"
    logger.info(f"Downloading {url} to {ffmpeg_folder}")
    req = requests.get(url, headers={"referer": "https://ffmpeg.zeranoe.com/"}, stream=True,)
    with open(ffmpeg_folder / "ffmpeg-latest-win64-static.zip", "wb") as f:
        for block in req.iter_content(chunk_size=4096):
            f.write(block)

    reusables.extract(ffmpeg_folder / "ffmpeg-latest-win64-static.zip", path=ffmpeg_folder)
    sub_dir = ffmpeg_folder / "ffmpeg-latest-win64-static"

    for item in os.listdir(sub_dir):
        shutil.move(str(sub_dir / item), str(ffmpeg_folder))

    try:
        sub_dir.unlink()
        Path("ffmpeg-latest-win64-static.zip").unlink()
    except OSError:
        pass


if __name__ == "__main__":
    try:
        main()
    except Exception:
        traceback.print_exc()
        input(
            "Error while running FastFlix!\n"
            "Plese report this issue on https://github.com/cdgriffith/FastFlix/issues (press any key to exit)"
        )
